import { AppPlugin } from '../types';
import { Asset, IPublicTypeProjectSchema } from '@alilc/lowcode-types';
import { injectComponents } from '@alilc/lowcode-plugin-inject';
//@ts-ignore
import { isArray, mergeWith } from 'lodash';
import { AssetLoader, buildComponents } from '@alilc/lowcode-utils';

function mergeArray(objValue: [], srcValue: []) {
  if (isArray(objValue)) {
    return objValue.concat(srcValue || []);
  }
  return undefined;
}

/**
 * schema转成component
 * @param schema
 * @param packages
 */
export async function parseSchema(
  schema: IPublicTypeProjectSchema,
  packages: any[]
) {
  const {
    componentsMap: componentsMapArray,
    componentsTree,
    i18n,
    dataSource: projectDataSource,
  } = schema;

  const componentsMap: any = {};
  componentsMapArray?.forEach((component: any) => {
    componentsMap[component.componentName] = component;
  });
  const pageSchema = componentsTree?.[0];

  const libraryMap: any = {};
  const libraryAsset: Asset[] = [];
  packages.forEach((pkg) => {
    const { package: _package, library, urls, renderUrls } = pkg;
    if (_package) {
      libraryMap[_package] = library;
    }
    if (renderUrls) {
      libraryAsset.push(renderUrls);
    } else if (urls) {
      libraryAsset.push(urls);
    }
  });

  const assetLoader = new AssetLoader();
  await assetLoader.load(libraryAsset);
  const components = await injectComponents(
    buildComponents(libraryMap, componentsMap, () => null)
  );

  return {
    schema: pageSchema,
    components,
    i18n,
    dataSource: mergeWith(
      pageSchema?.dataSource,
      projectDataSource,
      mergeArray
    ),
  };
}

export function loadPlugins(
  pluginsArray: Array<AppPlugin>,
  callback?: () => void,
  async?: boolean
) {
  const head = document.head;
  let loadedScriptsCount = 0;

  function onAllScriptLoad() {
    loadedScriptsCount++;
    // 当所有脚本都加载完成时，调用回调函数
    if (loadedScriptsCount === pluginsArray.length) {
      if (callback) {
        callback();
      }
    }
  }

  pluginsArray.forEach((plugin) => {
    if (!plugin.enable) {
      return;
    }
    const script = document.createElement('script');
    script.type = 'text/javascript';
    script.src = plugin.url;
    script.async = async ?? false; // 设置 async 属性以并行加载脚本
    script.onload = () => {
      if (plugin.callback) {
        try {
          const fn = new Function(plugin.callback);
          fn();
          onAllScriptLoad();
        } catch (e) {
          console.error(`Error excute callback: ${plugin.name} ${e}}`);
        }
      }
    };
    script.onerror = () => {
      console.error(`Error loading script: ${plugin.name}`);
    };
    head.appendChild(script);
  });
}
